==> plone.app.tiles-3.2.1-py2.py3-none-any.whl: plone/app/tiles/imagescaling.py <==
# -*- coding: utf-8 -*-
"""Image scale support for tile images."""
from Acquisition import aq_base
from ZODB.POSException import ConflictError
from logging import exception
from persistent.dict import PersistentDict
from plone.namedfile.interfaces import INamedImage
from plone.namedfile.scaling import ImageScale as BaseImageScale
from plone.namedfile.scaling import ImageScaling as BaseImageScaling
from plone.namedfile.utils import set_headers
from plone.namedfile.utils import stream_data
from plone.rfc822.interfaces import IPrimaryFieldInfo
from plone.scale.scale import scaleImage
from plone.scale.storage import AnnotationStorage as BaseAnnotationStorage
from plone.tiles.interfaces import ITileDataManager
from zope.interface import alsoProvides
from zope.publisher.interfaces import NotFound

try:
    from plone.protect.interfaces import IDisableCSRFProtection

    HAS_PLONE_PROTECT = True
except ImportError:
    HAS_PLONE_PROTECT = False

IMAGESCALES_KEY = '_plone.scales'


class AnnotationStorage(BaseAnnotationStorage):
    """ An abstract storage for image scale data using annotations and
        implementing :class:`IImageScaleStorage`. Image data is stored as an
        annotation on the object container, i.e. the image. This is needed
        since not all images are themselves annotatable. """

    @property
    def storage(self):
        tile = self.context
        manager = ITileDataManager(tile)
        data = manager.get()
        if IMAGESCALES_KEY not in data:
            data[IMAGESCALES_KEY] = PersistentDict()
            manager.set(data)
        return data[IMAGESCALES_KEY]


class ImageScale(BaseImageScale):
    """ view used for rendering image scales """

    def __init__(self, context, request, **info):
        self.context = context
        self.request = request
        self.__dict__.update(**info)
        if self.data is None:
            try:
                self.data = getattr(self.context, self.fieldname)
            except AttributeError:
                self.data = self.context.data[self.fieldname]

        url = self.context.url
        extension = self.data.contentType.split('/')[-1].lower()
        if 'uid' in info:
            name = info['uid']
        else:
            name = info['fieldname']
        self.__name__ = '%s.%s' % (name, extension)
        self.url = '%s/@@images/%s' % (url, self.__name__)

    def index_html(self):
        """ download the image """
        # validate access
        set_headers(self.data, self.request.response)
        return stream_data(self.data)


class ImageScaling(BaseImageScaling):
    """ view used for generating (and storing) image scales """

    def publishTraverse(self, request, name):
        """ used for traversal via publisher, i.e. when using as a url """
        stack = request.get('TraversalRequestNameStack')
        image = None
        if stack:
            # field and scale name were given...
            scale = stack.pop()
            image = self.scale(name, scale)  # this is aq-wrapped
        elif '-' in name:
            # we got a uid...
            if '.' in name:
                name, ext = name.rsplit('.', 1)
            storage = AnnotationStorage(self.context)
            info = storage.get(name)
            if info is not None:
                scale_view = ImageScale(self.context, self.request, **info)
                return scale_view
        else:
            # otherwise `name` must refer to a field...
            if '.' in name:
                name, ext = name.rsplit('.', 1)
            value = getattr(self.context, name)
            scale_view = ImageScale(self.context, self.request, data=value, fieldname=name)
            return scale_view
        if image is not None:
            return image
        raise NotFound(self, name, self.request)

    def create(self, fieldname, direction='thumbnail', height=None, width=None, **parameters):
        """ factory for image scales, see `IImageScaleStorage.scale` """
        orig_value = self.context.data.get(fieldname)
        if orig_value is None:
            return
        if height is None and width is None:
            _, format = orig_value.contentType.split('/', 1)
            return None, format, (orig_value._width, orig_value._height)
        if hasattr(aq_base(orig_value), 'open'):
            fp = orig_value.open()
            orig_data = fp.read()
            fp.close()
        else:
            orig_data = getattr(aq_base(orig_value), 'data', orig_value)
        if not orig_data:
            return
        try:
            result = scaleImage(
                orig_data, direction=direction, height=height, width=width, **parameters
            )
            # Disable Plone 5 implicit CSRF to allow scaling on GET
            if HAS_PLONE_PROTECT:
                alsoProvides(self.request, IDisableCSRFProtection)
        except (ConflictError, KeyboardInterrupt):
            raise
        except Exception:
            exception(
                'could not scale "%r" of %r', orig_value, self.context.context.absolute_url(),
            )
            return
        if result is not None:
            data, format, dimensions = result
            mimetype = 'image/%s' % format.lower()
            value = orig_value.__class__(data, contentType=mimetype, filename=orig_value.filename)
            value.fieldname = fieldname
            return value, format, dimensions

    def modified(self):
        """ provide a callable to return the modification time of content
            items, so stored image scales can be invalidated """
        mtime = 0
        for k, v in self.context.data.items():
            if INamedImage.providedBy(v):
                mtime += v._p_mtime
        return mtime

    def scale(self, fieldname=None, scale=None, height=None, width=None, **parameters):
        if fieldname is None:
            fieldname = IPrimaryFieldInfo(self.context).fieldname
        if scale is not None:
            available = self.getAvailableSizes()
            if scale not in available:
                return None
            width, height = available[scale]
        storage = AnnotationStorage(self.context, self.modified)
        info = storage.scale(
            factory=self.create, fieldname=fieldname, height=height, width=width, **parameters
        )
        if info is not None:
            info['fieldname'] = fieldname
            scale_view = ImageScale(self.context, self.request, **info)
            return scale_view

==> plone.app.tiles-3.2.2-py2.py3-none-any.whl: plone/app/tiles/imagescaling.py <==
# -*- coding: utf-8 -*-
"""Image scale support for tile images."""
from Acquisition import aq_base
from logging import exception
from persistent.dict import PersistentDict
from plone.namedfile.interfaces import INamedImage
from plone.namedfile.scaling import ImageScale as BaseImageScale
from plone.namedfile.scaling import ImageScaling as BaseImageScaling
from plone.namedfile.utils import set_headers
from plone.namedfile.utils import stream_data
from plone.rfc822.interfaces import IPrimaryFieldInfo
from plone.scale.scale import scaleImage
from plone.scale.storage import AnnotationStorage as BaseAnnotationStorage
from plone.tiles.interfaces import ITileDataManager
from ZODB.POSException import ConflictError
from zope.interface import alsoProvides
from zope.publisher.interfaces import NotFound


try:
    from plone.protect.interfaces import IDisableCSRFProtection

    HAS_PLONE_PROTECT = True
except ImportError:
    HAS_PLONE_PROTECT = False

IMAGESCALES_KEY = "_plone.scales"


class AnnotationStorage(BaseAnnotationStorage):
    """An abstract storage for image scale data using annotations and
    implementing :class:`IImageScaleStorage`. Image data is stored as an
    annotation on the object container, i.e. the image. This is needed
    since not all images are themselves annotatable."""

    @property
    def storage(self):
        tile = self.context
        manager = ITileDataManager(tile)
        data = manager.get()
        if IMAGESCALES_KEY not in data:
            data[IMAGESCALES_KEY] = PersistentDict()
            manager.set(data)
        return data[IMAGESCALES_KEY]


class ImageScale(BaseImageScale):
    """view used for rendering image scales"""

    def __init__(self, context, request, **info):
        self.context = context
        self.request = request
        self.__dict__.update(**info)
        if self.data is None:
            try:
                self.data = getattr(self.context, self.fieldname)
            except AttributeError:
                self.data = self.context.data[self.fieldname]

        url = self.context.url
        extension = self.data.contentType.split("/")[-1].lower()
        if "uid" in info:
            name = info["uid"]
        else:
            name = info["fieldname"]
        self.__name__ = "%s.%s" % (name, extension)
        self.url = "%s/@@images/%s" % (url, self.__name__)
        self.srcset = info.get("srcset", [])

    def index_html(self):
        """download the image"""
        # validate access
        set_headers(self.data, self.request.response)
        return stream_data(self.data)


class ImageScaling(BaseImageScaling):
    """view used for generating (and storing) image scales"""

    def publishTraverse(self, request, name):
        """used for traversal via publisher, i.e. when using as a url"""
        stack = request.get("TraversalRequestNameStack")
        image = None
        if stack:
            # field and scale name were given...
            scale = stack.pop()
            image = self.scale(name, scale)  # this is aq-wrapped
        elif "-" in name:
            # we got a uid...
            if "." in name:
                name, ext = name.rsplit(".", 1)
            storage = AnnotationStorage(self.context)
            info = storage.get(name)
            if info is not None:
                scale_view = ImageScale(self.context, self.request, **info)
                return scale_view
        else:
            # otherwise `name` must refer to a field...
            if "." in name:
                name, ext = name.rsplit(".", 1)
            value = getattr(self.context, name)
            scale_view = ImageScale(
                self.context, self.request, data=value, fieldname=name
            )
            return scale_view
        if image is not None:
            return image
        raise NotFound(self, name, self.request)

    def create(
        self, fieldname, direction="thumbnail", height=None, width=None, **parameters
    ):
        """factory for image scales, see `IImageScaleStorage.scale`"""
        orig_value = self.context.data.get(fieldname)
        if orig_value is None:
            return
        if height is None and width is None:
            _, format = orig_value.contentType.split("/", 1)
            return None, format, (orig_value._width, orig_value._height)
        if hasattr(aq_base(orig_value), "open"):
            fp = orig_value.open()
            orig_data = fp.read()
            fp.close()
        else:
            orig_data = getattr(aq_base(orig_value), "data", orig_value)
        if not orig_data:
            return
        try:
            result = scaleImage(
                orig_data, direction=direction, height=height, width=width, **parameters
            )
            # Disable Plone 5 implicit CSRF to allow scaling on GET
            if HAS_PLONE_PROTECT:
                alsoProvides(self.request, IDisableCSRFProtection)
        except (ConflictError, KeyboardInterrupt):
            raise
        except Exception:
            exception(
                'could not scale "%r" of %r',
                orig_value,
                self.context.context.absolute_url(),
            )
            return
        if result is not None:
            data, format, dimensions = result
            mimetype = "image/%s" % format.lower()
            value = orig_value.__class__(
                data, contentType=mimetype, filename=orig_value.filename
            )
            value.fieldname = fieldname
            return value, format, dimensions

    def modified(self):
        """provide a callable to return the modification time of content
        items, so stored image scales can be invalidated"""
        mtime = 0
        for k, v in self.context.data.items():
            if INamedImage.providedBy(v):
                mtime += v._p_mtime
        return mtime

    def scale(
        self,
        fieldname=None,
        scale=None,
        height=None,
        width=None,
        direction="thumbnail",
        **parameters
    ):
        if fieldname is None:
            fieldname = IPrimaryFieldInfo(self.context).fieldname
        if scale is not None:
            available = self.getAvailableSizes()
            if scale not in available:
                return None
            width, height = available[scale]
        storage = AnnotationStorage(self.context, self.modified)
        info = storage.scale(
            factory=self.create,
            fieldname=fieldname,
            height=height,
            width=width,
            direction=direction,
            **parameters
        )
        if info is not None:
            info["fieldname"] = fieldname
            scale_view = ImageScale(self.context, self.request, **info)
            return scale_view
